@@ -1,3 +1,11 @@
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
 #include "sirius.hpp"
 #include "testing.hpp"
 
@@ -18,30 +26,42 @@ test_enu(cmd_args const& args__)
         v[ir] = -double(zn) / rgrid[ir];
     }
 
-    Enu_finder e(rel, zn, n, l, rgrid, v, -0.1);
+    int auto_enu{1};
+
+    Enu_finder e(rel, zn, n, l, rgrid, v, -0.1, auto_enu);
+    auto enu_ref = e.enu();
+
+    #pragma omp parallel for
+    for (int i = 0; i < 100; i++) {
+        auto enu1 = Enu_finder(rel, zn, n, l, rgrid, v, -0.1, auto_enu).enu();
+        if (enu1 != enu_ref) {
+            std::cout << "wrong enu : " << enu1 << " " << enu_ref << std::endl;
+        }
+    }
 
     printf("Z: %i n: %i l: %i band energies (bottom, top, enu): %12.6f %12.6f %12.6f\n", zn, n, l, e.ebot(), e.etop(),
            e.enu());
 
     Radial_solver solver(zn, v, rgrid);
 
-    std::vector<double> p1, p2, p3, rdudr;
-    std::array<double, 2> uderiv;
-
     int dme{0};
 
-    solver.solve(rel, dme, l, e.ebot(), p1, rdudr, uderiv);
-    solver.solve(rel, dme, l, e.etop(), p2, rdudr, uderiv);
-    solver.solve(rel, dme, l, e.enu(), p3, rdudr, uderiv);
-
-    printf("uderiv: %12.6f %12.6f\n", uderiv[0], uderiv[1]);
+    auto result1 = solver.solve(rel, dme, l, e.ebot());
+    auto result2 = solver.solve(rel, dme, l, e.etop());
+    auto result3 = solver.solve(rel, dme, l, e.enu());
 
     FILE* fout = fopen("radial_solution.dat", "w");
     for (int i = 0; i < rgrid.num_points(); i++) {
         double x = rgrid[i];
-        fprintf(fout, "%18.12f %18.12f %18.12f %18.12f\n", x, p1[i] / x, p2[i] / x, p3[i] / x);
+        fprintf(fout, "%18.12f %18.12f %18.12f %18.12f\n", x, result1.p[i] / x, result2.p[i] / x, result3.p[i] / x);
     }
     fclose(fout);
+
+    auto enu2 = Enu_finder(rel, zn, n, l, rgrid, v, -0.1, auto_enu).enu();
+    auto enu3 = Enu_finder(rel, zn, n, l, rgrid, v, enu2, auto_enu).enu();
+
+    std::cout << "enu2: " << enu2 << ", enu3: " << enu3 << ", diff: " << std::abs(enu2 - enu3) << std::endl;
+
     return 0;
 }
 
@@ -55,7 +75,5 @@ main(int argn, char** argv)
                    {"n=", "(int) principal quantum number"},
                    {"R=", "(double) muffin-tin radius"}});
 
-    sirius::initialize(1);
-    call_test("test_enu", test_enu, args);
-    sirius::finalize();
+    return call_test("test_enu", test_enu, args);
 }
