@@ -1,10 +1,18 @@
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
 #include "sirius.hpp"
 #include "testing.hpp"
 
 using namespace sirius;
 
 int
-test1()
+test_gaunt_yyy()
 {
     int const lmax                      = 3;
     int const lmmax                     = (lmax + 1) * (lmax + 1);
@@ -292,9 +300,5 @@ test1()
 int
 main(int argn, char** argv)
 {
-    int err{0};
-
-    err += call_test("<Ylm|Ylm|Ylm>", test1);
-
-    return std::min(err, 1);
+    return call_test(argv[0], test_gaunt_yyy);
 }
