@@ -1,11 +1,20 @@
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
 #include <sirius.hpp>
+#include <testing.hpp>
 
 using namespace sirius;
 
-void
+int
 test1()
 {
-    int N = 400;
+    int N{400};
     matrix<std::complex<double>> A({N, N});
     matrix<std::complex<double>> B({N, N});
     matrix<std::complex<double>> C({N, N});
@@ -25,14 +34,16 @@ test1()
             .hemm('L', 'U', N, N, &la::constant<std::complex<double>>::one(), &A(0, 0), A.ld(), &B(0, 0), B.ld(),
                   &la::constant<std::complex<double>>::zero(), &C(0, 0), C.ld());
 
-    int err = 0;
+    int err{0};
     for (int i = 0; i < N; i++) {
         for (int j = 0; j < N; j++) {
             std::complex<double> z = C(i, j);
-            if (i == j)
+            if (i == j) {
                 z -= 1.0;
-            if (std::abs(z) > 1e-10)
+            }
+            if (std::abs(z) > 1e-10) {
                 err++;
+            }
         }
     }
 
@@ -42,26 +53,22 @@ test1()
     for (int i = 0; i < N; i++) {
         for (int j = 0; j < N; j++) {
             std::complex<double> z = C(i, j);
-            if (i == j)
+            if (i == j) {
                 z -= 1.0;
-            if (std::abs(z) > 1e-10)
+            }
+            if (std::abs(z) > 1e-10) {
                 err++;
+            }
         }
     }
-
-    if (err) {
-        printf("test1 failed!\n");
-        exit(1);
-    } else {
-        printf("test1 passed!\n");
-    }
+    return err;
 }
 
 template <typename T>
-void
+int
 test2()
 {
-    int N = 400;
+    int N{400};
     matrix<T> A({N, N});
     matrix<T> B({N, N});
     matrix<T> C({N, N});
@@ -77,7 +84,7 @@ test2()
             .gemm('N', 'N', N, N, N, &la::constant<T>::one(), A.at(memory_t::host), A.ld(), B.at(memory_t::host),
                   B.ld(), &la::constant<T>::zero(), C.at(memory_t::host), C.ld());
 
-    int err = 0;
+    int err{0};
     for (int i = 0; i < N; i++) {
         for (int j = 0; j < N; j++) {
             T c = C(i, j);
@@ -89,80 +96,77 @@ test2()
             }
         }
     }
+    return err;
+}
 
-    if (err) {
-        printf("test2 failed!\n");
-        exit(1);
-    } else {
-        printf("test2 passed!\n");
+/*
+#ifdef SIRIUS_SCALAPACK
+template <typename T>
+int test3()
+{
+    int bs = 32;
+
+    int num_ranks = mpi::Communicator::world().size();
+    int nrc = (int)std::sqrt(0.1 + num_ranks);
+    if (nrc * nrc != num_ranks) {
+        printf("wrong mpi grid\n");
+        exit(-1);
+    }
+
+    int N = 400;
+    la::BLACS_grid blacs_grid(mpi::Communicator::world(), nrc, nrc);
+
+    la::dmatrix<T> A(N, N, blacs_grid, bs, bs);
+    la::dmatrix<T> B(N, N, blacs_grid, bs, bs);
+    la::dmatrix<T> C(N, N, blacs_grid, bs, bs);
+    for (int i = 0; i < N; i++) {
+        for (int j = 0; j < N; j++) {
+            A.set(j, i, random<T>());
+        }
+    }
+    copy(A, B);
+
+    T alpha = 1.0;
+    T beta = 0.0;
+
+    la::wrap(la::lib_t::scalapack).geinv(N, A);
+
+    la::wrap(la::lib_t::scalapack).gemm('N', 'N', N, N, N, &alpha, A, 0, 0, B, 0, 0, &beta, C, 0, 0);
+
+    int err{0};
+    for (int i = 0; i < C.num_cols_local(); i++) {
+        for (int j = 0; j < C.num_rows_local(); j++) {
+            T c = C(j, i);
+            if (C.icol(i) == C.irow(j)) {
+                c -= 1.0;
+            }
+            if (std::abs(c) > 1e-10) {
+                err++;
+            }
+        }
     }
+    return err;
+}
+#endif
+*/
+
+int
+test_linalg()
+{
+    int err = test1();
+    err += test2<double>();
+    err += test2<std::complex<double>>();
+#ifdef SIRIUS_SCALAPACK
+    // err += test3<std::complex<double>>();
+#endif
+    return err;
 }
-// #ifdef SIRIUS_SCALAPACK
-// template <typename T>
-// void test3()
-//{
-//     int bs = 32;
-//
-//     int num_ranks = Communicator::world().size();
-//     int nrc = (int)std::sqrt(0.1 + num_ranks);
-//     if (nrc * nrc != num_ranks)
-//     {
-//         printf("wrong mpi grid\n");
-//         exit(-1);
-//     }
-//
-//     int N = 400;
-//     BLACS_grid blacs_grid(Communicator::world(), nrc, nrc);
-//
-//     dmatrix<T> A(N, N, blacs_grid, bs, bs);
-//     dmatrix<T> B(N, N, blacs_grid, bs, bs);
-//     dmatrix<T> C(N, N, blacs_grid, bs, bs);
-//     for (int i = 0; i < N; i++)
-//     {
-//         for (int j = 0; j < N; j++) A.set(j, i, utils::random<T>());
-//     }
-//     A >> B;
-//
-//     T alpha = 1.0;
-//     T beta = 0.0;
-//
-//     linalg(lib_t::scalapack).geinv(N, A);
-//
-//     linalg(lib_t::scalapack).gemm('N', 'N', N, N, N, &alpha, A, 0, 0, B, 0, 0, &beta, C, 0, 0);
-//
-//     int err = 0;
-//     for (int i = 0; i < C.num_cols_local(); i++)
-//     {
-//         for (int j = 0; j < C.num_rows_local(); j++)
-//         {
-//             T c = C(j, i);
-//             if (C.icol(i) == C.irow(j)) c -= 1.0;
-//             if (std::abs(c) > 1e-10) err++;
-//         }
-//     }
-//
-//     if (err)
-//     {
-//         printf("test3 failed!\n");
-//         exit(1);
-//     }
-//     else
-//     {
-//         printf("test3 passed!\n");
-//     }
-// }
-// #endif
 
 int
 main(int argn, char** argv)
 {
     sirius::initialize(1);
-    test1();
-    test2<double>();
-    test2<std::complex<double>>();
-    // #ifdef SIRIUS_SCALAPACK
-    // test3<std::complex<double>>();
-    // #endif
+    int result = call_test(argv[0], test_linalg);
     sirius::finalize();
-    return 0;
+    return result;
 }
