@@ -1,5 +1,13 @@
-#include "sirius.hpp"
-#include "testing.hpp"
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
+#include <sirius.hpp>
+#include <testing.hpp>
 
 using namespace sirius;
 
