@@ -1,3 +1,11 @@
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
+
 #ifndef __PYTHON_MODULE_INCLUDES_HPP__
 #define __PYTHON_MODULE_INCLUDES_HPP__
 
@@ -14,9 +22,9 @@
 #include <utility>
 #include <memory>
 #include <stdexcept>
-#ifdef _OPENMP
-#include <omp.h>
-#endif
+// #ifdef _OPENMP
+// #include <omp.h>
+// #endif
 
 #include "core/json.hpp"
 #include "dft/energy.hpp"
