@@ -1,40 +1,51 @@
+from __future__ import annotations
 import numpy as np
 from mpi4py import MPI
 from scipy.sparse import dia_matrix
-import collections
+from numpy.typing import ArrayLike, NDArray
+from typing import Union, Tuple, Any, Callable, get_type_hints, TypeAlias
+from collections import abc
+import copy
+
+scalar_t: TypeAlias = Union[int, float, complex, np.float64, np.complex128]
+
+
+def check_return_type(func: Callable) -> bool:
+    annotations = func.__annotations__
+    return annotations.get("return") == Tuple[Any, Any]
 
 
 def threaded(f):
-    """
-    decorator for threaded application over CoefficientArray
-    """
+    """Decorator for threaded application over CoefficientArray."""
+
     def _f(x, *args, **kwargs):
         if isinstance(x, CoefficientArray):
-            out = type(x)(dtype=x.dtype, ctype=x.ctype)
+            out = type(x)()
             for k in x._data.keys():
                 out[k] = f(x[k], *args, **kwargs)
             return out
         else:
             return f(x, *args, **kwargs)
+
     return _f
 
 
-def allthreaded(f):
-    def _f(*args):
-        out = type(args[0])(dtype=args[0].dtype, ctype=args[0].ctype)
-        for k in args[0]._data.keys():
-            out[k] = f(*map(lambda x: x.__getitem__(k), args))
-        return out
+def threaded_class(f):
+    """Decorator for threaded (in class members)
+    application over CoefficientArray."""
+
+    def _f(self, x, *args, **kwargs):
+        if isinstance(x, CoefficientArray):
+            out = type(x)()
+            for k in x._data.keys():
+                out[k] = f(self, x[k], *args, **kwargs)
+            return out
+        else:
+            return f(self, x, *args, **kwargs)
+
     return _f
 
 
-def is_complex(x):
-    if isinstance(x, CoefficientArray):
-        return np.iscomplexobj(x.dtype())
-    else:
-        return np.iscomplexobj(x)
-
-
 @threaded
 def sort(x):
     return np.sort(x)
@@ -55,13 +66,6 @@ def identity_like(x):
     return np.eye(*x.shape)
 
 
-def shapes(x):
-    shapes = {}
-    for k in x:
-        shapes[k] = np.shape(x[k])
-    return shapes
-
-
 def eye_like(shapes):
     """
     Arguments:
@@ -72,16 +76,16 @@ def eye_like(shapes):
     """
     out = CoefficientArray()
     for k in shapes:
-        out[k] = np.matrix(np.eye(*shapes[k]))
+        out[k] = np.eye(*shapes[k])
     return out
 
 
 def diag(x):
     """
-    TODO: make a check not to flatten a 2d matrix
+    TODO: add a check not to flatten a 2d matrix
     """
     if isinstance(x, CoefficientArray):
-        out = type(x)(dtype=x.dtype, ctype=x.ctype)
+        out = type(x)()
         for key, val in x._data.items():
             if val.size == max(val.shape):
                 out[key] = np.diag(np.array(val, copy=True).flatten())
@@ -97,7 +101,7 @@ def spdiag(x):
     Diagonal matrix (scipy.sparse.dia_matrix)
     """
     if isinstance(x, CoefficientArray):
-        out = type(x)(dtype=x.dtype, ctype=x.ctype)
+        out = type(x)()
         for key, val in x._data.items():
             n = np.size(val)
             out[key] = dia_matrix((val, 0), shape=(n, n))
@@ -106,22 +110,18 @@ def spdiag(x):
     return dia_matrix((x, 0), shape=(n, n))
 
 
-def ones_like(x, dtype=None):
+def ones_like(x, *args):
     """Numpy ones_like."""
     if isinstance(x, CoefficientArray):
-        return CoefficientArray.ones_like(x, dtype=dtype)
-    if dtype is None:
-        dtype = x.dtype
-    return np.ones_like(x, dtype=dtype)
+        return CoefficientArray.ones_like(x, *args)
+    return np.ones_like(x, *args)
 
 
-def zeros_like(x, dtype=None):
+def zeros_like(x, *args):
     """Numpy zeros_like."""
     if isinstance(x, CoefficientArray):
-        return CoefficientArray.zeros_like(x, dtype=dtype)
-    if dtype is None:
-        dtype = x.dtype
-    return np.zeros_like(x, dtype=dtype)
+        return CoefficientArray.zeros_like(x, *args)
+    return np.zeros_like(x, *args)
 
 
 def inner(a, b):
@@ -130,7 +130,7 @@ def inner(a, b):
 
 
 def l2norm(a):
-    """L2-norm. """
+    """L2-norm."""
     return np.sqrt(np.real(inner(a, a)))
 
 
@@ -144,45 +144,31 @@ def einsum(expr, *operands):
     try:
         return np.einsum(expr, *operands)
     except (ValueError, TypeError):
-        out = type(operands[0])(dtype=operands[0].dtype, ctype=np.array)
+        out = type(operands[0])()
         for key in operands[0]._data.keys():
-            out[key] = np.einsum(expr,
-                                 *list(map(lambda x: x[key], operands)))
+            out[key] = np.einsum(expr, *list(map(lambda x: x[key], operands)))
         return out
 
 
-class CoefficientArrayBase(collections.abc.Mapping):
-
+class CoefficientArrayBase(abc.Mapping):
     def __init__(self):
         self._data = {}
 
     def __iter__(self):
         yield from self._data
 
-    def __getitem__(self, key):
-        return self._data[key]
-
-    def __setitem__(self, key, value):
-        self._data[key] = value
-
     def __len__(self):
         return len(self._data)
-    # # workaround for numpy exploiting the __iter__(self)
-    # __array_priority__ = 10000
 
 
 class CoefficientArray(CoefficientArrayBase):
     """CoefficientArray class."""
-    def __init__(self, dtype=np.complex128, ctype=np.matrix):
-        """
-        dtype -- number type
-        ctype -- container type (default np.matrix)
-        """
-        super().__init__()
-        self.dtype = dtype
-        self.ctype = ctype
 
-    def __getitem__(self, key):
+    def __init__(self):
+        """ """
+        super().__init__()
+
+    def __getitem__(self, key: Tuple[int, int]):
         """
         key -- (k, ispn)
         """
@@ -193,27 +179,31 @@ class CoefficientArray(CoefficientArrayBase):
         # return as view
         return self._data[key]
 
-    def __setitem__(self, key, item):
-        """
-        """
-        if isinstance(key, slice):
-            for k in self._data:
-                self._data[key] = item
+    def __setitem__(
+        self, key: Tuple[int, int], item: Union[ArrayLike, dia_matrix, Any]
+    ):
+        """ """
+        if (key in self._data) and hasattr(item, "__array__"):
+            x = self._data[key]
+            # make sure shapes don't change
+            x[:] = item
         else:
-            if key in self._data:
-                x = self._data[key]
-                # make sure shapes don't change
-                x[:] = item
+            if isinstance(item, np.ndarray):
+                self._data[key] = copy.deepcopy(item)
             else:
-                if isinstance(item, np.ndarray):
-                    self._data[key] = self.ctype(item, dtype=self.dtype, copy=False)
-                else:
-                    self._data[key] = item
+                self._data[key] = item
 
-    def __array_ufunc__(self, ufunc, method, *inputs, **kwargs):
-        if ufunc not in (np.sqrt, np.log, np.absolute, np.conjugate, np.multiply, np.divide):
+    def __array_ufunc__(self, ufunc, method, *inputs, **_):
+        if ufunc not in (
+            np.sqrt,
+            np.log,
+            np.absolute,
+            np.conjugate,
+            np.multiply,
+            np.divide,
+        ):
             return NotImplemented
-        if method == '__call__':
+        if method == "__call__":
             if ufunc == np.sqrt:
                 return self.sqrt()
             elif ufunc == np.log:
@@ -232,107 +222,91 @@ class CoefficientArray(CoefficientArrayBase):
                     return NotImplemented
                 else:
                     return self.__rtruediv__(inputs[0])
-
             else:
                 return NotImplemented
         else:
             return NotImplemented
 
     def sum(self, **kwargs):
-        """
-        """
-        loc_sum = np.array(
-            sum([np.sum(v) for _, v in self.items()]), dtype=self.dtype)
+        """ """
+        loc_sum = np.array(sum([np.sum(v, **kwargs) for _, v in self.items()]))
         reduced = MPI.COMM_WORLD.allreduce(loc_sum, op=MPI.SUM)
         return np.ndarray.item(reduced)
 
     def log(self, **kwargs):
-        out = type(self)(dtype=self.dtype, ctype=self.ctype)
+        out = type(self)()
         for key in self._data.keys():
             out[key] = np.log(self._data[key], **kwargs)
         return out
 
     def __mul__(self, other):
-        """
-        """
-        if is_complex(self) or is_complex(other):
-            dtype = np.complex128
-        else:
-            dtype = np.float64
-
-        out = type(self)(dtype=dtype, ctype=self.ctype)
+        """ """
+        out = type(self)()
         if isinstance(other, CoefficientArray):
             for key in other._data.keys():
-                out[key] = np.einsum('...,...->...', self._data[key],
-                                     other._data[key])
+                out[key] = np.einsum("...,...->...", self._data[key], other._data[key])
         elif np.isscalar(other):
             for key in self._data.keys():
                 out[key] = self._data[key] * other
         else:
-            raise TypeError('wrong type')
+            raise TypeError("wrong type")
         return out
 
-    def __truediv__(self, other):
-        if is_complex(self) or is_complex(other):
-            dtype = np.complex128
+    def max(self, **kwargs):
+        if kwargs:
+            return [np.max(self._data[key], **kwargs) for key in self._data.keys()]
         else:
-            dtype = np.float64
+            return np.max([np.max(self._data[key]) for key in self._data.keys()])
 
-        out = type(self)(dtype=dtype, ctype=self.ctype)
+    def min(self, **kwargs):
+        if kwargs:
+            return [np.min(self._data[key], **kwargs) for key in self._data.keys()]
+        else:
+            return np.min([np.min(self._data[key]) for key in self._data.keys()])
+
+    def __truediv__(self, other):
+        out = type(self)()
         if isinstance(other, CoefficientArray):
             for key in other._data.keys():
-                out[key] = np.einsum('...,...->...', self._data[key],
-                                     1 / other._data[key])
+                out[key] = np.einsum(
+                    "...,...->...", self._data[key], 1 / other._data[key]
+                )
         elif np.isscalar(other):
             for key in self._data.keys():
                 out[key] = self._data[key] / other
         else:
-            raise TypeError('wrong type')
+            raise TypeError("wrong type")
         return out
 
     def __rtruediv__(self, other):
-        if is_complex(self) or is_complex(other):
-            dtype = np.complex128
-        else:
-            dtype = np.float64
-
-        out = type(self)(dtype=dtype, ctype=self.ctype)
+        out = type(self)()
         if isinstance(other, CoefficientArray):
             for key in other._data.keys():
-                out[key] = np.einsum('...,...->...', 1 / self._data[key],
-                                     other._data[key])
+                out[key] = np.einsum(
+                    "...,...->...", 1 / self._data[key], other._data[key]
+                )
         elif np.isscalar(other):
             for key in self._data.keys():
                 out[key] = other / self._data[key]
         else:
-            raise TypeError('wrong type')
+            raise TypeError("wrong type")
         return out
 
-    def __matmul__(self, other):
-        """
-        """
-        if is_complex(self) or is_complex(other):
-            dtype = np.complex128
-        else:
-            dtype = np.float64
-        out = type(self)(dtype=dtype)
+    def __matmul__(self, other: CoefficientArrayBase):
+        """ """
+        out = type(self)()
         if isinstance(other, CoefficientArray):
             for key in other._data.keys():
                 out[key] = self._data[key] @ other._data[key]
         elif np.isscalar(other):
             for key in self._data.keys():
-                out[key] = self._data[key] @ other
+                out[key] = self._data[key] * other
         else:
-            raise TypeError('wrong type')
+            raise TypeError("wrong type")
         return out
 
-    def __add__(self, other):
-        if is_complex(self) or is_complex(other):
-            dtype = np.complex128
-        else:
-            dtype = np.float64
-
-        out = type(self)(dtype=dtype, ctype=self.ctype)
+    def __add__(self, other: Union[CoefficientArrayBase, scalar_t]) -> CoefficientArray:
+        out = type(self)()
         if isinstance(other, CoefficientArray):
             for key in other._data.keys():
                 out[key] = self._data[key] + other._data[key]
@@ -341,20 +315,20 @@ class CoefficientArray(CoefficientArrayBase):
                 out[key] = self._data[key] + other
         return out
 
-    def __neg__(self):
-        out = type(self)(dtype=self.dtype, ctype=self.ctype)
+    def __neg__(self) -> CoefficientArray:
+        out = type(self)()
         for key, val in self._data.items():
             out[key] = -val
         return out
 
-    def abs(self):
-        out = type(self)(dtype=np.float64, ctype=self.ctype)
+    def abs(self) -> CoefficientArray:
+        out = type(self)()
         for key in self._data.keys():
             out[key] = np.abs(self._data[key])
         return out
 
-    def sqrt(self):
-        out = type(self)(dtype=self.dtype, ctype=self.ctype)
+    def sqrt(self) -> CoefficientArray:
+        out = type(self)()
         for key in self._data.keys():
             out[key] = np.sqrt(self._data[key])
         return out
@@ -363,9 +337,9 @@ class CoefficientArray(CoefficientArrayBase):
         """
         returns U, s, Vh
         """
-        U = type(self)(dtype=self.dtype, ctype=self.ctype)
-        s = type(self)(dtype=np.float64, ctype=np.array)
-        Vh = type(self)(dtype=self.dtype, ctype=self.ctype)
+        U = type(self)()
+        s = type(self)()
+        Vh = type(self)()
         for key in self._data.keys():
             Ul, sl, Vhl = np.linalg.svd(self[key], **args)
             U[key] = Ul
@@ -374,22 +348,22 @@ class CoefficientArray(CoefficientArrayBase):
         return U, s, Vh
 
     def eigh(self, **args):
-        w = type(self)(dtype=np.float64, ctype=np.array)
-        V = type(self)(dtype=self.dtype, ctype=np.matrix)
+        w = type(self)()
+        V = type(self)()
         for key in self._data.keys():
             w[key], V[key] = np.linalg.eigh(self[key], **args)
         return w, V
 
     def eig(self, **args):
-        w = type(self)(dtype=np.complex128, ctype=np.array)
-        V = type(self)(dtype=self.dtype, ctype=np.matrix)
+        w = type(self)()
+        V = type(self)()
         for key in self._data.keys():
             w[key], V[key] = np.linalg.eig(self[key], **args)
         return w, V
 
     def qr(self, **args):
-        Q = type(self)(dtype=self.dtype, ctype=np.matrix)
-        R = type(self)(dtype=self.dtype, ctype=np.matrix)
+        Q = type(self)()
+        R = type(self)()
         for key in self._data.keys():
             Q[key], R[key] = np.linalg.qr(self[key], **args)
         return Q, R
@@ -397,81 +371,63 @@ class CoefficientArray(CoefficientArrayBase):
     def keys(self):
         return self._data.keys()
 
-    def __sub__(self, other):
+    def __sub__(self, other) -> CoefficientArray:
         return self.__add__(-1 * other)
 
-    def __rsub__(self, other):
+    def __rsub__(self, other) -> CoefficientArray:
         return -(self.__add__(-1 * other))
 
     def __pow__(self, a):
-        """
-
-        """
-        out = type(self)(dtype=self.dtype, ctype=self.ctype)
+        """ """
+        out = type(self)()
         for key in self._data.keys():
-            out[key] = self._data[key]**a
+            out[key] = self._data[key] ** a
         return out
 
-    def conjugate(self):
-        out = type(self)(dtype=self.dtype, ctype=self.ctype)
+    def conjugate(self) -> CoefficientArray:
+        out = type(self)()
         for key, val in self._data.items():
             out[key] = np.array(np.conj(val), copy=True)
         return out
 
-    def conj(self):
+    def conj(self) -> CoefficientArray:
         return self.conjugate()
 
-    def flatten(self, ctype=None):
-        if ctype is None:
-            ctype = self.ctype
-        out = type(self)(dtype=self.dtype, ctype=ctype)
+    def flatten(self, *args):
+        out = type(self)()
         for key, val in self._data.items():
-            out[key] = ctype(val).flatten()
+            out[key] = val.flatten(*args)
         return out
 
-    def asarray(self):
-        """
-
-        """
-        out = type(self)(dtype=self.dtype, ctype=np.array)
+    def asarray(self) -> CoefficientArray:
+        """ """
+        out = type(self)()
         for key, val in self._data.items():
-            out[key] = np.array(val)
-        return out
-
-    def cols(self, indices):
-        """
-
-        """
-        out = type(self)(dtype=self.dtype, ctype=self.ctype)
-        for k, jj in indices._data.items():
-            out[k] = self[k][:, jj]
+            out[key] = np.asarray(val)
         return out
 
     def __len__(self):
-        """
-        """
+        """ """
         return len(self._data)
 
     def items(self):
-        """
-        """
+        """ """
         return self._data.items()
 
     def __contains__(self, key):
-        """
-        """
-        return self._data.__contains(key)
+        """ """
+        return self._data.__contains__(key)
 
     @property
     def real(self):
-        out = type(self)(dtype=np.float64)
+        out = type(self)()
         for key, val in self._data.items():
             out[key] = np.real(val)
         return out
 
     @property
     def imag(self):
-        out = type(self)(dtype=np.float64)
+        out = type(self)()
         for key, val in self._data.items():
             out[key] = np.imag(val)
         return out
@@ -481,9 +437,9 @@ class CoefficientArray(CoefficientArrayBase):
         """
         Hermitian conjugate
         """
-        out = type(self)(dtype=self.dtype, ctype=self.ctype)
+        out = type(self)()
         for key, val in self._data.items():
-            out[key] = np.atleast_2d(val).H
+            out[key] = np.conj(np.atleast_2d(val)).T
         return out
 
     @property
@@ -491,24 +447,22 @@ class CoefficientArray(CoefficientArrayBase):
         """
         Transpose
         """
-        out = type(self)(dtype=self.dtype, ctype=self.ctype)
+        out = type(self)()
         for key, val in self._data.items():
             out[key] = np.atleast_2d(val).T
         return out
 
     def __str__(self):
-        return '\n'.join([
-            '\n'.join(map(str, [key, val, '---']))
-            for key, val in self._data.items()
-        ])
+        return "\n".join(
+            ["\n".join(map(str, [key, val, "---"])) for key, val in self._data.items()]
+        )
 
-    def to_array(self):
+    def to_array(self) -> NDArray:
         """
         convert to numpy array
         """
         if len(self) > 0:
-            return np.concatenate(list(map(np.atleast_1d, self._data.values())),
-                                  axis=0)
+            return np.concatenate(list(map(np.atleast_1d, self._data.values())), axis=0)
         else:
             return np.array([])
 
@@ -518,42 +472,34 @@ class CoefficientArray(CoefficientArrayBase):
         assumes that each entry has the same number of columns
         """
         offset = 0
-        for key, val in self._data.items():
-            val[:] = X[offset:offset + val.shape[0], ...]
+        for _, val in self._data.items():
+            val[:] = X[offset : offset + val.shape[0], ...]
             offset += val.shape[0]
 
-    def _repr_pretty_(self, p, cycle):
+    def _repr_pretty_(self, p, *_):
         for key, val in self._data.items():
-            p.text('key: ')
+            p.text("key: ")
             p.pretty(key)
-            p.text('\n')
+            p.text("\n")
             p.pretty(val)
-            p.text('\n')
+            p.text("\n")
 
     def __repr__(self):
         for key in self._data:
             print(key, self._data[key])
 
     @staticmethod
-    def ones_like(x, dtype=None, ctype=None):
-        if ctype is None:
-            ctype = x.ctype
-        if dtype is None:
-            dtype = x.dtype
-        out = type(x)(dtype=dtype, ctype=ctype)
+    def ones_like(x, *args):
+        out = type(x)()
         for k in x._data.keys():
-            out[k] = np.ones_like(x[k], dtype=dtype)
+            out[k] = np.ones_like(x[k], *args)
         return out
 
     @staticmethod
-    def zeros_like(x, dtype=None, ctype=None):
-        if ctype is None:
-            ctype = x.ctype
-        if dtype is None:
-            dtype = x.dtype
-        out = type(x)(dtype=dtype, ctype=ctype)
+    def zeros_like(x, *args):
+        out = type(x)()
         for k in x._data.keys():
-            out[k] = np.zeros_like(x[k], dtype=dtype)
+            out[k] = np.zeros_like(x[k], *args)
         return out
 
     __lmul__ = __mul__
@@ -563,8 +509,8 @@ class CoefficientArray(CoefficientArrayBase):
 
 
 class PwCoeffs(CoefficientArray):
-    def __init__(self, kpointset=None, dtype=np.complex128, ctype=np.matrix):
-        super().__init__(dtype=dtype, ctype=ctype)
+    def __init__(self, kpointset=None):
+        super().__init__()
 
         # load plane-wave coefficients from kpointset
         if kpointset is not None:
@@ -573,7 +519,9 @@ class PwCoeffs(CoefficientArray):
                 k = kpointset[ki]
                 for ispn in range(num_sc):
                     key = ki, ispn
-                    val = np.matrix(np.array(k.spinor_wave_functions().pw_coeffs(ispn), order='F', copy=True), copy=False)
+                    val = np.array(
+                        k.spinor_wave_functions().pw_coeffs(ispn), order="F", copy=True
+                    )
                     self.__setitem__(key, val)
 
     def __setitem__(self, key, item):
@@ -581,15 +529,15 @@ class PwCoeffs(CoefficientArray):
         key -- (k, ispn)
         """
         # return as view
-        assert (len(key) == 2)
+        assert len(key) == 2
         return super(PwCoeffs, self).__setitem__(key, item)
 
     def kview(self, k):
-        """
-        """
-        out = PwCoeffs(dtype=self.dtype)
-        out._data = {(ki, ispn): self._data[(ki, ispn)]
-                     for ki, ispn in self._data if ki == k}
+        """ """
+        out = PwCoeffs()
+        out._data = {
+            (ki, ispn): self._data[(ki, ispn)] for ki, ispn in self._data if ki == k
+        }
         return out
 
     def kvalues(self):
@@ -613,15 +561,44 @@ class PwCoeffs(CoefficientArray):
         return sdict
 
 
-if __name__ == '__main__':
-    shapes = [(10, 12), (3, 100), (4, 80), (5, 60)]
+def allthreaded(f, return_type=PwCoeffs):
+    """Decorator for threaded application over CoefficientArray. In contrast
+    to `threaded`, all positional arguments of f are "threaded".
+    """
+    type_hints = get_type_hints(f)
+    return_type_hint = type_hints["return"]
+    nret = 1
+    if hasattr(return_type_hint, "__origin__") and return_type_hint.__origin__ is tuple:
+        nret = len(return_type_hint.__args__)
+
+    def _f(*args):
+        if nret > 1:
+            # f is returning a tuple
+            out = tuple(return_type() for _ in range(nret))
+            for k in args[0]._data.keys():
+                tmp = f(*map(lambda x: x.__getitem__(k), args))
+                for out_n, tmp_n in zip(out, tmp):
+                    out_n[k] = tmp_n
+            return out
+        else:
+            # f is returning a single object
+            out = return_type()
+            for k in args[0]._data.keys():
+                out[k] = f(*map(lambda x: x.__getitem__(k), args))
+            return out
+
+    return _f
+
+
+if __name__ == "__main__":
+    shapes_ = [(10, 12), (3, 100), (4, 80), (5, 60)]
     keys = [(1, 0), (1, 1), (2, 0), (2, 1)]
 
     CC = PwCoeffs()
 
-    for k, sh in zip(keys, shapes):
-        print('k:', k)
-        print('sh:', sh)
+    for k, sh in zip(keys, shapes_):
+        print("k:", k)
+        print("sh:", sh)
         CC[k] = np.random.rand(*sh)
 
     # scale
@@ -630,10 +607,10 @@ if __name__ == '__main__':
     CC = 2 + CC + 1
     CC = CC - 1j
     CC = np.conj(CC)
-    print('np.conj(CC) has type: ', type(CC))
+    print("np.conj(CC) has type: ", type(CC))
     # not working
     # CC = abs(CC)
 
     CCv = CC.kview(1)
     for k, item in CC.by_k().items():
-        print('list of k:', k, 'has ', len(item), ' entries')
+        print("list of k:", k, "has ", len(item), " entries")
