@@ -20,7 +20,7 @@
 #include <cublas_v2.h>
 
 #elif defined(SIRIUS_ROCM)
-#include <rocblas.h>
+#include <rocblas/rocblas.h>
 
 #else
 #error Either SIRIUS_CUDA or SIRIUS_ROCM must be defined!
