@@ -70,7 +70,7 @@ inline auto
 get_rho_up_dn(int num_mag_dims__, double rho__, r3::vector<double> mag__)
 {
     if (rho__ < 0.0) {
-        return std::make_pair<double, double>(0.0, 0.0);
+        return std::make_pair(0.0, 0.0);
     }
 
     double mag{0};
@@ -85,7 +85,7 @@ get_rho_up_dn(int num_mag_dims__, double rho__, r3::vector<double> mag__)
         mag = std::min(mag__.length(), rho__);
     }
 
-    return std::make_pair<double, double>(0.5 * (rho__ + mag), 0.5 * (rho__ - mag));
+    return std::make_pair(0.5 * (rho__ + mag), 0.5 * (rho__ - mag));
 }
 
 /// PAW density storage.
@@ -244,14 +244,21 @@ class Density : public Field4D
                                  Periodic_function<double>, density_matrix_t, PAW_density<double>, Hubbard_matrix>>
             mixer_;
 
+    /// Core charge density.
+    /** All-electron core charge density of the LAPW method. It is recomputed on every SCF iteration due to
+        the change of effective potential. */
+    std::vector<std::vector<double>> ae_core_charge_density_;
+
+    /// Core eigen-value sum.
+    std::vector<double> core_eval_sum_;
+
+    /// Core leakage.
+    std::vector<double> core_leakage_;
+
     /// Generate atomic densities in the case of PAW.
     void
     generate_paw_density(paw_atom_index_t::local iapaw__);
 
-    /// Initialize PAW density matrix.
-    void
-    init_density_matrix_for_paw();
-
     /// Reduce complex density matrix over magnetic quantum numbers
     /** The following operation is performed:
         \f[
@@ -300,45 +307,15 @@ class Density : public Field4D
     /// Add k-point contribution to the density and magnetization defined on the regular FFT grid.
     template <typename T>
     void
-    add_k_point_contribution_rg(K_point<T>* kp__, std::array<wf::Wave_functions_fft<T>, 2>& wf_fft__);
+    add_k_point_contribution_rg(K_point<T>* kp__, std::array<wf::Wave_functions_fft<T>, 2> const& wf_fft__);
 
     /// Generate valence density in the muffin-tins
     void
     generate_valence_mt();
 
-    /// Generate charge density of core states
+    /// Generate pseudo core charge density needed for non-linear core correction.
     void
-    generate_core_charge_density()
-    {
-        PROFILE("sirius::Density::generate_core_charge_density");
-
-        auto& spl_idx = unit_cell_.spl_num_atom_symmetry_classes();
-
-        for (auto it : spl_idx) {
-            unit_cell_.atom_symmetry_class(it.i).generate_core_charge_density(ctx_.core_relativity());
-        }
-
-        for (auto ic = begin_global(spl_idx); ic != end_global(spl_idx); ic++) {
-            auto rank = spl_idx.location(ic).ib;
-            unit_cell_.atom_symmetry_class(ic).sync_core_charge_density(ctx_.comm(), rank);
-        }
-    }
-
-    void
-    generate_pseudo_core_charge_density()
-    {
-        PROFILE("sirius::Density::generate_pseudo_core_charge_density");
-
-        /* get lenghts of all G shells */
-        auto q = ctx_.gvec().shells_len();
-        /* get form-factors for all G shells */
-        auto const ff = ctx_.ri().ps_core_->values(q, ctx_.comm());
-        /* make rho_core(G) */
-        auto v = make_periodic_function<true>(ctx_.unit_cell(), ctx_.gvec(), ctx_.phase_factors_t(), ff);
-
-        std::copy(v.begin(), v.end(), &rho_pseudo_core_->f_pw_local(0));
-        rho_pseudo_core_->fft_transform(1);
-    }
+    generate_pseudo_core_charge_density();
 
   public:
     /// Constructor
@@ -352,6 +329,10 @@ class Density : public Field4D
     double
     core_leakage() const;
 
+    /// Find total sum of core eigen-values.
+    double
+    core_eval_sum() const;
+
     /// Generate initial charge density and magnetization
     void
     initial_density();
@@ -369,6 +350,10 @@ class Density : public Field4D
     bool
     check_num_electrons() const;
 
+    /// Generate charge density of core states
+    void
+    generate_core_charge_density(std::vector<std::vector<double>> const& vs__);
+
     /// Generate full charge density (valence + core) and magnetization from the wave functions.
     /** This function calls generate_valence() and then in case of full-potential LAPW method adds a core density
         to get the full charge density of the system. Density is generated in spectral representation, i.e.
@@ -418,10 +403,16 @@ class Density : public Field4D
     generate_rho_aug() const;
 
     /// Return core leakage for a specific atom symmetry class
-    inline double
+    inline auto
     core_leakage(int ic) const
     {
-        return unit_cell_.atom_symmetry_class(ic).core_leakage();
+        return core_leakage_[ic];
+    }
+
+    inline auto
+    core_eval_sum(int ic) const
+    {
+        return core_eval_sum_[ic];
     }
 
     /// Return const reference to charge density (scalar functions).
@@ -600,37 +591,10 @@ class Density : public Field4D
     }
 
     void
-    save(std::string name__) const
-    {
-        rho().hdf5_write(name__, "density");
-        for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-            std::stringstream s;
-            s << "magnetization/" << j;
-            mag(j).hdf5_write(name__, s.str());
-        }
-        ctx_.comm().barrier();
-    }
+    save(std::string name__) const;
 
     void
-    load(std::string name__)
-    {
-        HDF5_tree fin(name__, hdf5_access_t::read_only);
-
-        int ngv;
-        fin.read("/parameters/num_gvec", &ngv, 1);
-        if (ngv != ctx_.gvec().num_gvec()) {
-            RTE_THROW("wrong number of G-vectors");
-        }
-        mdarray<int, 2> gv({3, ngv});
-        fin.read("/parameters/gvec", gv);
-
-        rho().hdf5_read(name__, "density", gv);
-        rho().rg().fft_transform(1);
-        for (int j = 0; j < ctx_.num_mag_dims(); j++) {
-            mag(j).hdf5_read(name__, "magnetization/" + std::to_string(j), gv);
-            mag(j).rg().fft_transform(1);
-        }
-    }
+    load(std::string name__);
 
     void
     save_to_xsf()
