@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2018 Anton Kozhevnikov, Mathieu Taillefumier, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file hamiltonian_k.cpp
  *
@@ -100,7 +89,7 @@ Hamiltonian_k<T>::get_h_o_diag_pw() const
         #pragma omp parallel for schedule(static)
         for (int ig_loc = 0; ig_loc < kp_.num_gkvec_loc(); ig_loc++) {
             if (what & 1) {
-                auto ekin            = 0.5 * kp_.gkvec().template gkvec_cart<index_domain_t::local>(ig_loc).length2();
+                auto ekin            = 0.5 * kp_.gkvec().gkvec_cart(gvec_index_t::local(ig_loc)).length2();
                 h_diag(ig_loc, ispn) = ekin + H0_.local_op().v0(ispn);
             }
             if (what & 2) {
@@ -217,7 +206,7 @@ Hamiltonian_k<T>::get_h_o_diag_lapw() const
     #pragma omp parallel for schedule(static)
     for (int igloc = 0; igloc < kp_.num_gkvec_loc(); igloc++) {
         if (what & 1) {
-            auto gvc      = kp_.gkvec().template gkvec_cart<index_domain_t::local>(igloc);
+            auto gvc      = kp_.gkvec().gkvec_cart(gvec_index_t::local(igloc));
             T ekin        = 0.5 * dot(gvc, gvc);
             h_diag[igloc] = H0_.local_op().v0(0) + ekin * H0_.ctx().theta_pw(0).real();
         }
@@ -434,7 +423,8 @@ Hamiltonian_k<T>::set_fv_h_o(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std:
                 }
 
                 kp_.alm_coeffs_col().template generate<false>(atom, alm_col_atom);
-                /* can't copy alm to device how as it might be modified by the iora */
+
+                /* can't copy alm to device now as it might be modified by the iora */
 
                 H0_.apply_hmt_to_apw(atom, spin_block_t::nm, kp_.num_gkvec_col(), alm_col_atom, halm_col_atom);
                 if (pu == device_t::GPU) {
@@ -510,7 +500,7 @@ Hamiltonian_k<T>::set_fv_h_o(la::dmatrix<std::complex<T>>& h__, la::dmatrix<std:
     /* setup lo-lo block */
     set_fv_h_o_lo_lo(h__, o__);
 
-    ///*  copy back to GPU */ // TODO: optimize the copies
+    /*  copy back to GPU */ // TODO: optimize the copies
     // if (pu == device_t::GPU) {
     //     acc::copyin(h__.at(memory_t::device), h__.ld(), h__.at(memory_t::host), h__.ld(), kp.gklo_basis_size_row(),
     //         kp.gklo_basis_size_col());
@@ -667,24 +657,23 @@ Hamiltonian_k<T>::set_fv_h_o_it(la::dmatrix<std::complex<T>>& h__, la::dmatrix<s
     #pragma omp parallel for default(shared)
     for (int igk_col = 0; igk_col < kp.num_gkvec_col(); igk_col++) {
         /* fractional coordinates of G vectors */
-        auto gvec_col = kp.gkvec_col().template gvec<index_domain_t::local>(igk_col);
+        auto gvec_col = kp.gkvec_col().gvec(gvec_index_t::local(igk_col));
         /* Cartesian coordinates of G+k vectors */
-        auto gkvec_col_cart = kp.gkvec_col().template gkvec_cart<index_domain_t::local>(igk_col);
+        auto gkvec_col_cart = kp.gkvec_col().gkvec_cart(gvec_index_t::local(igk_col));
         for (int igk_row = 0; igk_row < kp.num_gkvec_row(); igk_row++) {
-            auto gvec_row       = kp.gkvec_row().template gvec<index_domain_t::local>(igk_row);
-            auto gkvec_row_cart = kp.gkvec_row().template gkvec_cart<index_domain_t::local>(igk_row);
+            auto gvec_row       = kp.gkvec_row().gvec(gvec_index_t::local(igk_row));
+            auto gkvec_row_cart = kp.gkvec_row().gkvec_cart(gvec_index_t::local(igk_row));
             int ig12            = H0().ctx().gvec().index_g12(gvec_row, gvec_col);
-            /* pw kinetic energy */
-            double t1 = 0.5 * r3::dot(gkvec_row_cart, gkvec_col_cart);
 
             h__(igk_row, igk_col) += H0().potential().veff_pw(ig12);
             o__(igk_row, igk_col) += H0().ctx().theta_pw(ig12);
 
+            /* pw kinetic energy */
+            double t1 = 0.5 * r3::dot(gkvec_row_cart, gkvec_col_cart);
+
             switch (H0().ctx().valence_relativity()) {
                 case relativity_t::iora: {
-                    h__(igk_row, igk_col) += t1 * H0().potential().rm_inv_pw(ig12);
                     o__(igk_row, igk_col) += t1 * sq_alpha_half * H0().potential().rm2_inv_pw(ig12);
-                    break;
                 }
                 case relativity_t::zora: {
                     h__(igk_row, igk_col) += t1 * H0().potential().rm_inv_pw(ig12);
