@@ -181,14 +181,35 @@ Hubbard_matrix::print_local(int at_lvl__, std::ostream& out__) const
 
     auto print_number = [&](double x) { out__ << std::setw(width) << std::setprecision(prec) << std::fixed << x; };
     auto const& atom  = ctx_.unit_cell().atom(atomic_orbitals_[at_lvl__].first);
+    auto solver       = la::Eigensolver_factory("lapack");
 
-    out__ << "level : " << atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl__].second).n();
+    out__ << "atom : " << atom.type().label();
+    out__ << " level : " << atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl__].second).n();
     out__ << " l: " << atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl__].second).l() << std::endl;
     const int l = atom.type().lo_descriptor_hub(atomic_orbitals_[at_lvl__].second).l();
     if (ctx_.num_mag_dims() != 3) {
         int mmax = 2 * l + 1;
+        std::vector<double> eigenvals(mmax);
+        la::dmatrix<double> eigenvecs(mmax, mmax);
+        la::dmatrix<double> A(mmax, mmax);
         for (int is = 0; is < ctx_.num_spins(); is++) {
-            out__ << hbar(width * mmax, '-') << std::endl;
+            for (int m = 0; m < mmax; m++) {
+                for (int mp = 0; mp < mmax; mp++) {
+                    A(m, mp) = std::real(this->local(at_lvl__)(m, mp, is));
+                }
+            }
+            solver->solve(mmax, A, &eigenvals[0], eigenvecs);
+            // don't print "SPIN: 1" for non-magnetic case
+            if (ctx_.num_spins() == 1) {
+                out__ << hbar(width * mmax, '-') << std::endl;
+            } else {
+                out__ << hbar(width * mmax, '-') << " SPIN: " << is + 1 << std::endl;
+            }
+            // print eigenvalues
+            for (const auto& val : eigenvals) {
+                print_number(val);
+            }
+            out__ << std::endl << hbar(width * mmax, '-') << std::endl;
             bool has_imag{false};
             for (int m = 0; m < mmax; m++) {
                 for (int mp = 0; mp < mmax; mp++) {
@@ -323,11 +344,11 @@ Hubbard_matrix::print_nonlocal(int idx__, std::ostream& out__) const
 void
 Hubbard_matrix::zero()
 {
-    for (int ia = 0; ia < static_cast<int>(local_.size()); ia++) {
-        local_[ia].zero();
+    for (int i = 0; i < static_cast<int>(local_.size()); i++) {
+        local_[i].zero();
     }
 
-    for (int i = 0; i < static_cast<int>(ctx_.cfg().hubbard().nonlocal().size()); i++) {
+    for (int i = 0; i < static_cast<int>(nonlocal_.size()); i++) {
         nonlocal_[i].zero();
     }
 
@@ -338,4 +359,16 @@ Hubbard_matrix::zero()
     }
 }
 
+void
+Hubbard_matrix::print(std::ostream& out__) const
+{
+    for (int i = 0; i < static_cast<int>(local_.size()); i++) {
+        this->print_local(i, out__);
+    }
+
+    for (int i = 0; i < static_cast<int>(nonlocal_.size()); i++) {
+        this->print_nonlocal(i, out__);
+    }
+}
+
 } // namespace sirius
