@@ -6,10 +6,13 @@
  * SPDX-License-Identifier: BSD-3-Clause
  */
 
+#include <fmt/base.h>
 #include <limits>
+#include "core/rte/rte.hpp"
 #include "dft/smearing.hpp"
 #include "k_point/k_point.hpp"
 #include "k_point/k_point_set.hpp"
+#include "core/expected.hpp"
 #include "symmetry/get_irreducible_reciprocal_mesh.hpp"
 #include <iomanip>
 
@@ -167,7 +170,7 @@ K_point_set::initialize(std::vector<int> const& counts)
 }
 
 template <class F>
-double
+util::expected<local::efermi_search_t, std::string>
 bisection_search(F&& f, double a, double b, double tol, int maxstep = 1000)
 {
     double x  = (a + b) / 2;
@@ -187,14 +190,12 @@ bisection_search(F&& f, double a, double b, double tol, int maxstep = 1000)
         fi = f(x);
 
         if (step > maxstep) {
-            std::stringstream s;
-            s << "search of band occupancies failed after 10000 steps";
-            RTE_THROW(s);
+            return util::unexpected(fmt::format("search for chemical potential failed after {} steps", maxstep));
         }
         step++;
     }
 
-    return x;
+    return local::efermi_search_t{.mu = x, .ne_diff = std::abs(fi), .iter = step};
 }
 
 /**
@@ -209,30 +210,23 @@ bisection_search(F&& f, double a, double b, double tol, int maxstep = 1000)
  *  \param  maxstep max number of Newton iterations
  */
 template <class Nt, class DNt, class D2Nt>
-auto
-newton_minimization_chemical_potential(Nt&& N, DNt&& dN, D2Nt&& ddN, double mu0, double ne, double tol,
+util::expected<local::efermi_search_t, std::string>
+newton_minimization_chemical_potential(Nt&& N, DNt&& dN, D2Nt&& ddN, double mu0, double ne, double tol, double tol_ne,
                                        int maxstep = 1000)
 {
     // Newton finds the minimum, not necessarily N(mu) == ne, tolerate up to `tol_ne` difference in number of electrons
     // if |N(mu_0) -ne| > tol_ne an error is thrown.
-    const double tol_ne = 1e-2;
-
-    struct
-    {
-        double mu;              // chemical potential
-        int iter{0};            // newton information
-        std::vector<double> ys; // newton history
-    } res;
-
+    // const double tol_ne = 1e-10;
     double mu = mu0;
     double alpha{1.0}; // Newton damping
     int iter{0};
 
-    if (std::abs(N(mu) - ne) < tol) {
-        res.mu   = mu;
-        res.iter = iter;
-        res.ys   = {};
-        return res;
+    if (double ne_diff = std::abs(N(mu) - ne); ne_diff < tol) {
+        return local::efermi_search_t{
+                .mu      = mu,
+                .ne_diff = ne_diff,
+                .iter    = iter,
+        };
     }
 
     while (true) {
@@ -242,41 +236,36 @@ newton_minimization_chemical_potential(Nt&& N, DNt&& dN, D2Nt&& ddN, double mu0,
         double ddNf = ddN(mu);
         /* minimize (N(mu) - ne)^2  */
         // double F = (Nf - ne) * (Nf - ne);
-        double dF   = 2 * (Nf - ne) * dNf;
-        double ddF  = 2 * dNf * dNf + 2 * (Nf - ne) * ddNf;
-        double step = alpha * dF / std::abs(ddF);
-        mu          = mu - step;
-
-        res.ys.push_back(mu);
+        double dF  = 2 * (Nf - ne) * dNf;
+        double ddF = 2 * dNf * dNf + 2 * (Nf - ne) * ddNf;
 
         if (std::abs(ddF) < 1e-30) {
             std::stringstream s;
-            s << "Newton minimization (Fermi energy) failed because 2nd derivative too close to zero!"
-              << std::setprecision(8) << std::abs(Nf - ne) << "\n";
-            RTE_THROW(s);
+            return util::unexpected(
+                    "Newton minimization (Fermi energy) failed because 2nd derivative too close to zero!");
         }
 
+        double step = alpha * dF / std::abs(ddF);
+        mu          = mu - step;
+
         if (std::abs(step) < tol || std::abs(Nf - ne) < tol) {
             if (std::abs(Nf - ne) > tol_ne) {
-                std::stringstream s;
-                s << "Newton minimization (Fermi energy) got stuck in a local minimum. Fallback to bisection search."
-                  << "\n";
-                RTE_THROW(s);
+                return util::unexpected("Newton minimization (Fermi energy) got stuck in a local minimum");
             }
-
-            res.iter = iter;
-            res.mu   = mu;
-            return res;
+            return local::efermi_search_t{
+                    .mu      = mu,
+                    .ne_diff = std::abs(Nf - ne),
+                    .iter    = iter,
+            };
         }
 
         iter++;
         if (iter > maxstep) {
-            std::stringstream s;
-            s << "Newton minimization (chemical potential) failed after " << maxstep << " steps!" << std::endl
-              << "target number of electrons : " << ne << std::endl
-              << "initial guess for chemical potential : " << mu0 << std::endl
-              << "current value of chemical potential : " << mu;
-            RTE_THROW(s);
+            return util::unexpected(fmt::format("Newton minimization (chemical potential) failed after {} steps\n"
+                                                "Target #electrons  :  {}\n"
+                                                "Initial guess for μ:  {}\n"
+                                                "Current value for μ: {}",
+                                                maxstep, ne, mu0, mu));
         }
     }
 }
@@ -309,22 +298,30 @@ K_point_set::find_band_occupancies_without_empty()
     this->sync_band<T, sync_band_t::occupancy>();
 }
 
+/**
+ * Find Fermi energy for fixed magnetization.
+ * \param emin   min band energy
+ * \param emax   max band energy
+ * \return A tuple containing
+           - efermi
+           - Fermi energy corrections for spin up [0] and down [1]
+ */
 template <typename T>
-void
-K_point_set::find_band_occupancies_fixed_magn(double emin, double emax)
+std::tuple<double, std::array<double, 2>>
+K_point_set::find_efermi_fixed_magn(double emin, double emax) const
 {
     /* split number of bands between available ranks */
     splindex_block<> splb(ctx_.num_bands(), n_blocks(ctx_.comm_band().size()), block_id(ctx_.comm_band().rank()));
-
     double const ne_target = ctx_.unit_cell().num_valence_electrons() - ctx_.cfg().parameters().extra_charge();
-
     double const fixed_mag = ctx_.cfg().parameters().fixed_mag();
+    auto f                 = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
 
+    double occ[2]   = {(ne_target + fixed_mag) / 2, (ne_target - fixed_mag) / 2};
     auto compute_ne = [&](int ispn, double ef, auto&& f) {
         double ne{0};
         for (auto it : spl_num_kpoints_) {
             double tmp{0};
-            #pragma omp parallel for reduction(+:tmp)
+            #pragma omp parallel for reduction(+ : tmp)
             for (int j = 0; j < splb.local_size(); j++) {
                 tmp += f(ef - this->get<T>(it.i)->band_energy(splb.global_index(j), ispn)) * ctx_.max_occupancy();
             }
@@ -334,41 +331,33 @@ K_point_set::find_band_occupancies_fixed_magn(double emin, double emax)
         return ne;
     };
 
-    auto f = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
-
-    double occ[2] = {(ne_target + fixed_mag) / 2, (ne_target - fixed_mag) / 2};
-    double ef[2];
+    std::array<double, 2> ef;
 
     for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
-        auto F   = [&compute_ne, ispn, occ, &f](double x) { return compute_ne(ispn, x, f) - occ[ispn]; };
-        ef[ispn] = bisection_search(F, emin, emax, 1e-11);
-    }
-
-    energy_fermi_ = std::max(ef[0], ef[1]);
-
-    /* compute occupations */
-    for (auto it : spl_num_kpoints_) {
-        for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
-            #pragma omp parallel for
-            for (int j = 0; j < ctx_.num_bands(); j++) {
-                auto o = f(ef[ispn] - this->get<T>(it.i)->band_energy(j, ispn)) * ctx_.max_occupancy();
-                this->get<T>(it.i)->band_occupancy(j, ispn, o);
-            }
+        auto F      = [&compute_ne, ispn, occ, &f](double x) { return compute_ne(ispn, x, f) - occ[ispn]; };
+        auto result = bisection_search(F, emin, emax, 1e-11);
+        if (!result) {
+            RTE_THROW(result.error());
         }
+        ef[ispn] = result.value().mu;
     }
+
+    double energy_fermi = std::max(ef[0], ef[1]);
+
+    std::array<double, 2> ef_corr = {ef[0] - energy_fermi, ef[1] - energy_fermi};
+
+    return std::make_tuple(energy_fermi, ef_corr);
 }
 
 template <typename T>
-void
-K_point_set::find_band_occupancies_generic(double emin, double emax)
+local::efermi_search_t
+K_point_set::find_efermi_generic(double emin, double emax) const
 {
     double const tol{1e-11};
 
     /* split number of bands between available ranks */
     splindex_block<> splb(ctx_.num_bands(), n_blocks(ctx_.comm_band().size()), block_id(ctx_.comm_band().rank()));
-
     double const ne_target = ctx_.unit_cell().num_valence_electrons() - ctx_.cfg().parameters().extra_charge();
-
     /* computes N(ef; f) = \sum_{i,k} f(ef - e_{k,i}) */
     auto compute_ne = [&](double ef, auto&& f) {
         double ne{0};
@@ -388,63 +377,65 @@ K_point_set::find_band_occupancies_generic(double emin, double emax)
     };
 
     /* smearing function */
-    std::function<double(double)> f;
-    if (ctx_.smearing() == smearing::smearing_t::cold || ctx_.smearing() == smearing::smearing_t::methfessel_paxton) {
-        // obtain initial guess for non-monotous smearing with Gaussian
-        f = [&](double x) { return smearing::gaussian::occupancy(x, ctx_.smearing_width()); };
-    } else {
-        f = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
-    }
-
-    try {
-        auto F        = [&compute_ne, ne_target, &f](double x) { return compute_ne(x, f) - ne_target; };
-        energy_fermi_ = bisection_search(F, emin, emax, 1e-11);
-
-        /* for cold and Methfessel Paxton smearing start newton minimization  */
-        if (ctx_.smearing() == smearing::smearing_t::cold ||
-            ctx_.smearing() == smearing::smearing_t::methfessel_paxton) {
-            f               = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
-            auto df         = smearing::delta(ctx_.smearing(), ctx_.smearing_width());
-            auto ddf        = smearing::dxdelta(ctx_.smearing(), ctx_.smearing_width());
-            auto N          = [&](double mu) { return compute_ne(mu, f); };
-            auto dN         = [&](double mu) { return compute_ne(mu, df); };
-            auto ddN        = [&](double mu) { return compute_ne(mu, ddf); };
-            auto res_newton = newton_minimization_chemical_potential(N, dN, ddN, energy_fermi_, ne_target, tol, 1000);
-            energy_fermi_   = res_newton.mu;
-            if (ctx_.verbosity() >= 2) {
-                RTE_OUT(ctx_.out()) << "newton iteration converged after " << res_newton.iter << " steps\n";
+    switch (ctx_.smearing()) {
+        // non-monotic smearing types
+        case smearing::smearing_t::cold:
+        case smearing::smearing_t::methfessel_paxton: {
+            auto fgauss = [&](double x) { return smearing::gaussian::occupancy(x, ctx_.smearing_width()); };
+            auto F      = [&compute_ne, ne_target, &fgauss](double x) { return compute_ne(x, fgauss) - ne_target; };
+            auto bgauss_init = bisection_search(F, emin, emax, 1e-11);
+            if (!bgauss_init) {
+                RTE_THROW(bgauss_init.error());
             }
-        }
-    } catch (std::exception const& e) {
-        if (ctx_.verbosity() >= 2) {
-            RTE_OUT(ctx_.out()) << e.what() << std::endl << "fallback to bisection search" << std::endl;
-        }
-        f             = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
-        auto F        = [&compute_ne, ne_target, &f](double x) { return compute_ne(x, f) - ne_target; };
-        energy_fermi_ = bisection_search(F, emin, emax, tol);
-    }
+            double efermi_guess             = bgauss_init.value().mu;
+            std::function<double(double)> f = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
+            auto df                         = smearing::delta(ctx_.smearing(), ctx_.smearing_width());
+            auto ddf                        = smearing::dxdelta(ctx_.smearing(), ctx_.smearing_width());
+            auto N                          = [&](double mu) { return compute_ne(mu, f); };
+            auto dN                         = [&](double mu) { return compute_ne(mu, df); };
+            auto ddN                        = [&](double mu) { return compute_ne(mu, ddf); };
+            auto res_newton = newton_minimization_chemical_potential(N, dN, ddN, efermi_guess, ne_target, tol,
+                                                                     ctx_.cfg().settings().tol_ne(), 1000);
+            if (res_newton) {
+                // Newton search success
+                if (ctx_.verbosity() >= 2) {
+                    RTE_OUT(ctx_.out()) << "Newton iteration converged after " << res_newton.value().iter << " steps\n";
+                }
+                return res_newton.value();
+            } // use bisection search
 
-    for (auto it : spl_num_kpoints_) {
-        for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
-            #pragma omp parallel for
-            for (int j = 0; j < ctx_.num_bands(); j++) {
-                auto o = f(energy_fermi_ - this->get<T>(it.i)->band_energy(j, ispn)) * ctx_.max_occupancy();
-                this->get<T>(it.i)->band_occupancy(j, ispn, o);
+            auto bsearch = bisection_search(
+                    [&compute_ne, ne_target, &f](double x) { return compute_ne(x, f) - ne_target; }, emin, emax, 1e-11);
+            if (!bsearch) {
+                RTE_THROW(bsearch.error());
             }
+            return bsearch.value();
+        }
+        // all monotic smearing types
+        default: {
+            std::function<double(double)> f = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
+            auto M      = [&compute_ne, ne_target, &f](double x) { return compute_ne(x, f) - ne_target; };
+            auto result = bisection_search(M, emin, emax, 1e-11);
+            if (!result) {
+                RTE_THROW(result.error());
+            }
+            return result.value();
         }
     }
 }
 
 template <typename T>
-void
+double
 K_point_set::find_band_occupancies()
 {
     PROFILE("sirius::K_point_set::find_band_occupancies");
 
+    double ne_diff{0};
+
     auto band_occ_callback = ctx_.band_occ_callback();
     if (band_occ_callback) {
         band_occ_callback();
-        return;
+        return ne_diff;
     }
 
     /* target number of electrons */
@@ -453,7 +444,7 @@ K_point_set::find_band_occupancies()
     /* this is a special case when there are no empty states */
     if (ctx_.num_mag_dims() != 1 && std::abs(ctx_.num_bands() * ctx_.max_occupancy() - ne_target) < 1e-10) {
         this->find_band_occupancies_without_empty<T>();
-        return;
+        return ne_diff;
     }
 
     if (ctx_.smearing_width() == 0) {
@@ -464,7 +455,7 @@ K_point_set::find_band_occupancies()
     auto emin = std::numeric_limits<double>::max();
     auto emax = std::numeric_limits<double>::lowest();
 
-    #pragma omp parallel for reduction(min:emin) reduction(max:emax)
+    #pragma omp parallel for reduction(min : emin) reduction(max : emax)
     for (auto it : spl_num_kpoints_) {
         for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
             emin = std::min(emin, this->get<T>(it.i)->band_energy(0, ispn));
@@ -477,15 +468,32 @@ K_point_set::find_band_occupancies()
     /* split number of bands between available ranks */
     splindex_block<> splb(ctx_.num_bands(), n_blocks(ctx_.comm_band().size()), block_id(ctx_.comm_band().rank()));
 
-    /* collinear case with fixed magenetisation */
+    std::array<double, 2> ef_corr{0, 0};
     if (std::abs(ctx_.cfg().parameters().fixed_mag()) > 1e-10 && ctx_.num_mag_dims() == 1) {
-        this->find_band_occupancies_fixed_magn<T>(emin, emax);
+        // collinear case with fixed magenetisation
+        std::tie(energy_fermi_, ef_corr) = this->find_efermi_fixed_magn<T>(emin, emax);
     } else {
-        this->find_band_occupancies_generic<T>(emin, emax);
+        // generic case
+        auto res_efermi = find_efermi_generic<T>(emin, emax);
+        energy_fermi_   = res_efermi.mu;
+        ne_diff         = res_efermi.ne_diff;
+    }
+    /* set band occupancies */
+    auto f = smearing::occupancy(ctx_.smearing(), ctx_.smearing_width());
+    for (auto it : spl_num_kpoints_) {
+        for (int ispn = 0; ispn < ctx_.num_spins(); ispn++) {
+            #pragma omp parallel for
+            for (int j = 0; j < ctx_.num_bands(); j++) {
+                auto o = f(this->energy_fermi_ + ef_corr[ispn] - this->get<T>(it.i)->band_energy(j, ispn)) *
+                         ctx_.max_occupancy();
+                this->get<T>(it.i)->band_occupancy(j, ispn, o);
+            }
+        }
     }
 
     this->sync_band<T, sync_band_t::occupancy>();
 
+    /* compute band gap */
     band_gap_ = 0.0;
 
     int nve = static_cast<int>(ne_target + 1e-12);
@@ -520,12 +528,13 @@ K_point_set::find_band_occupancies()
             band_gap_ = eband[ist].first - eband[ist - 1].second;
         }
     }
+    return ne_diff;
 }
 
-template void
+template double
 K_point_set::find_band_occupancies<double>();
 #if defined(SIRIUS_USE_FP32)
-template void
+template double
 K_point_set::find_band_occupancies<float>();
 #endif
 
@@ -540,7 +549,7 @@ K_point_set::valence_eval_sum() const
     for (auto it : spl_num_kpoints_) {
         auto const& kp = this->get<T>(it.i);
         double tmp{0};
-        #pragma omp parallel for reduction(+:tmp)
+        #pragma omp parallel for reduction(+ : tmp)
         for (int j = 0; j < splb.local_size(); j++) {
             for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
                 tmp += kp->band_energy(splb.global_index(j), ispn) * kp->band_occupancy(splb.global_index(j), ispn);
@@ -589,7 +598,7 @@ K_point_set::entropy_sum() const
     for (auto it : spl_num_kpoints_) {
         auto const& kp = this->get<T>(it.i);
         double tmp{0};
-        #pragma omp parallel for reduction(+:tmp)
+        #pragma omp parallel for reduction(+ : tmp)
         for (int j = 0; j < splb.local_size(); j++) {
             for (int ispn = 0; ispn < ctx_.num_spinors(); ispn++) {
                 tmp += ctx_.max_occupancy() * f(energy_fermi_ - kp->band_energy(splb.global_index(j), ispn));
@@ -690,8 +699,7 @@ K_point_set::load()
     //==     for (int ik = 0; ik < num_kpoints(); ik++)
     //==     {
     //==         r3::vector<double> dvk;
-    //==         for (int x = 0; x < 3; x++) dvk[x] = vk_in[x] - kpoints_[ik]->vk(x);
-    //==         if (dvk.length() < 1e-12)
+    //==         for (int x = 0; x < 3; x++) dvk[x] = vk_in[x] - kpoints_[ik]->vk(x);//==         if (dvk.length() < 1e-12)
     //==         {
     //==             ikidx[ik] = jk;
     //==             break;
