@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2019 Simon Frasch, Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file broyden2_mixer.hpp
  *
@@ -98,13 +87,15 @@ namespace mixer {
 template <typename... FUNCS>
 class Broyden2 : public Mixer<FUNCS...>
 {
+  private:
+    double beta_;
+    mdarray<double, 2> S_;
+    mdarray<double, 1> gamma_;
+
   public:
-    Broyden2(std::size_t max_history, double beta, double beta0, double beta_scaling_factor, double linear_mix_rmse_tol)
+    Broyden2(std::size_t max_history, double beta)
         : Mixer<FUNCS...>(max_history)
         , beta_(beta)
-        , beta0_(beta0)
-        , beta_scaling_factor_(beta_scaling_factor)
-        , linear_mix_rmse_tol_(linear_mix_rmse_tol)
         , S_({max_history, max_history})
         , gamma_({max_history})
     {
@@ -185,15 +176,8 @@ class Broyden2 : public Mixer<FUNCS...>
             }
         }
     }
-
-  private:
-    double beta_;
-    double beta0_;
-    double beta_scaling_factor_;
-    double linear_mix_rmse_tol_;
-    mdarray<double, 2> S_;
-    mdarray<double, 1> gamma_;
 };
+
 } // namespace mixer
 } // namespace sirius
 
