@@ -1,21 +1,10 @@
-// Copyright (c) 2013-2017 Anton Kozhevnikov, Thomas Schulthess
-// All rights reserved.
-//
-// Redistribution and use in source and binary forms, with or without modification, are permitted provided that
-// the following conditions are met:
-//
-// 1. Redistributions of source code must retain the above copyright notice, this list of conditions and the
-//    following disclaimer.
-// 2. Redistributions in binary form must reproduce the above copyright notice, this list of conditions
-//    and the following disclaimer in the documentation and/or other materials provided with the distribution.
-//
-// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
-// WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
-// PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
-// ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
-// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
-// CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
-// OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+/* This file is part of SIRIUS electronic structure library.
+ *
+ * Copyright (c), ETH Zurich.  All rights reserved.
+ *
+ * Please, refer to the LICENSE file in the root directory.
+ * SPDX-License-Identifier: BSD-3-Clause
+ */
 
 /** \file radial_grid.hpp
  *
@@ -27,6 +16,7 @@
 
 #include "core/memory.hpp"
 #include "core/typedefs.hpp"
+#include "core/rte/rte.hpp"
 
 namespace sirius {
 
@@ -196,7 +186,7 @@ class Radial_grid
         return dx_;
     }
 
-    Radial_grid<T>
+    auto
     segment(int num_points__) const
     {
         assert(num_points__ >= 0 && num_points__ <= (int)x_.size());
@@ -213,7 +203,7 @@ class Radial_grid
         return r;
     }
 
-    std::vector<real_type<T>>
+    auto
     values() const
     {
         std::vector<real_type<T>> v(num_points());
@@ -359,7 +349,7 @@ Radial_grid_factory(radial_grid_t grid_type__, int num_points__, T rmin__, T rma
             break;
         }
         default: {
-            throw std::runtime_error("wrong radial grid type");
+            RTE_THROW("wrong radial grid type");
         }
     }
     return rgrid;
@@ -372,7 +362,7 @@ get_radial_grid_t(std::string str__)
     if (pos == std::string::npos) {
         std::stringstream s;
         s << "wrong string for the radial grid type: " << str__;
-        throw std::runtime_error(s.str());
+        RTE_THROW(s);
     }
 
     std::string name = str__.substr(0, pos);
