@@ -14,7 +14,9 @@
 #include "atom_type.hpp"
 #include "core/ostream_tools.hpp"
 #include "core/traits.hpp"
+#include "potential/xc_functional_base.hpp"
 #include <algorithm>
+#include <vector>
 
 namespace sirius {
 
@@ -217,6 +219,39 @@ Atom_type::init()
             free_atom_density_[i] = std::exp(-x) * zn_ / 8 / pi;
         }
     }
+    /* restrore free atom potential from spherical density */
+    if (parameters_.full_potential()) {
+        init_free_atom_density(false);
+        free_atom_potential_spline_ = Spline<double>(free_atom_radial_grid_);
+        XC_functional_base Ex("XC_LDA_X", 1);
+        XC_functional_base Ec("XC_LDA_C_VWN", 1);
+        std::vector<double> g1, g2;
+        /* compute Hartree potential */
+        free_atom_density_spline_.integrate(g2, 2);
+        double t1 = free_atom_density_spline_.integrate(g1, 1);
+
+        auto np = free_atom_radial_grid_.num_points();
+
+        std::vector<double> vh(np);
+        /* electrostatic potential of electrons */
+        for (int i = 0; i < np; i++) {
+            vh[i] = fourpi * (g2[i] / free_atom_radial_grid_.x(i) + t1 - g1[i]);
+        }
+
+        std::vector<double> vx(np);
+        std::vector<double> vc(np);
+        std::vector<double> ex(np);
+        std::vector<double> ec(np);
+
+        /* compute XC potential and energy */
+        Ex.get_lda(np, &free_atom_density_[0], &vx[0], &ex[0]);
+        Ec.get_lda(np, &free_atom_density_[0], &vc[0], &ec[0]);
+
+        for (int i = 0; i < np; i++) {
+            free_atom_potential_spline_(i) = vh[i] - this->zn() * free_atom_radial_grid_.x_inv(i) + vc[i] + vx[i];
+        }
+        free_atom_potential_spline_.interpolate();
+    }
 
     if (parameters_.full_potential()) {
         using gc_z   = Gaunt_coefficients<std::complex<double>>;
@@ -1142,24 +1177,21 @@ Atom_type::read_hubbard_input()
     for (int i = 0; i < parameters_.cfg().hubbard().local().size(); i++) {
         auto ho = parameters_.cfg().hubbard().local(i);
         if (ho.atom_type() == this->label()) {
-            std::array<double, 6> coeff{0, 0, 0, 0, 0, 0};
-            if (ho.contains("U")) {
-                coeff[0] = ho.U();
-            }
+            std::array<double, 3> coeff;
+            std::fill(coeff.begin(), coeff.end(), 0);
+            double alpha = 0.0;
+            double beta  = 0.0;
             if (ho.contains("J")) {
-                coeff[1] = ho.J();
+                coeff[0] = ho.J();
             }
-            if (ho.contains("BE2")) {
-                coeff[2] = ho.BE2();
+            if (ho.contains("E2")) {
+                coeff[1] = ho.E2();
             }
             if (ho.contains("E3")) {
-                coeff[3] = ho.E3();
+                coeff[2] = ho.E3();
             }
-            if (ho.contains("alpha")) {
-                coeff[4] = ho.alpha();
-            }
-            if (ho.contains("beta")) {
-                coeff[5] = ho.beta();
+            if (ho.contains("B")) {
+                coeff[1] = ho.B();
             }
 
             std::vector<double> initial_occupancy;
@@ -1176,8 +1208,8 @@ Atom_type::read_hubbard_input()
                 }
             }
 
-            add_hubbard_orbital(ho.n(), ho.l(), ho.total_initial_occupancy(), coeff[0], coeff[1], &coeff[0], coeff[4],
-                                coeff[5], 0.0, initial_occupancy, true);
+            add_hubbard_orbital(ho.n(), ho.l(), ho.total_initial_occupancy(), ho.U(), coeff[0], coeff, alpha, beta, 0.0,
+                                initial_occupancy, true);
 
             this->hubbard_correction_ = true;
         }
@@ -1190,7 +1222,9 @@ Atom_type::read_hubbard_input()
                 auto& e  = ps_atomic_wfs_[s];
                 int n    = e.n;
                 auto aqn = e.am;
-                add_hubbard_orbital(n, aqn.l(), 0, 0, 0, nullptr, 0, 0, 0.0, std::vector<double>(2 * aqn.l() + 1, 0),
+                std::array<double, 3> hub_coef;
+                std::fill(hub_coef.begin(), hub_coef.end(), 0);
+                add_hubbard_orbital(n, aqn.l(), 0, 0, 0, hub_coef, 0, 0, 0.0, std::vector<double>(2 * aqn.l() + 1, 0),
                                     false);
             }
         } else {
@@ -1199,12 +1233,14 @@ Atom_type::read_hubbard_input()
                 int n    = e.n;
                 auto aqn = e.am;
 
-                // check if the orbital is already listed. In that case skip it
+                /* check if the orbital is already listed. In that case skip it */
                 for (int i = 0; i < parameters_.cfg().hubbard().local().size(); i++) {
                     auto ho = parameters_.cfg().hubbard().local(i);
                     if ((ho.atom_type() == this->label()) && ((ho.n() != n) || (ho.l() != aqn.l()))) {
-                        // we add it to the list but we only use it for the orthogonalization procedure
-                        add_hubbard_orbital(n, aqn.l(), 0, 0, 0, nullptr, 0, 0, 0.0,
+                        std::array<double, 3> hub_coeff;
+                        std::fill(hub_coeff.begin(), hub_coeff.end(), 0);
+                        /* we add it to the list but we only use it for the orthogonalization procedure */
+                        add_hubbard_orbital(n, aqn.l(), 0, 0, 0, hub_coeff, 0, 0, 0.0,
                                             std::vector<double>(2 * aqn.l() + 1, 0), false);
                         break;
                     }
@@ -1215,7 +1251,7 @@ Atom_type::read_hubbard_input()
 }
 
 void
-Atom_type::add_hubbard_orbital(int n__, int l__, double occ__, double U, double J, const double* hub_coef__,
+Atom_type::add_hubbard_orbital(int n__, int l__, double occ__, double U, double J, std::array<double, 3> hub_coef__,
                                double alpha__, double beta__, double J0__, std::vector<double> initial_occupancy__,
                                const bool use_for_calculations__)
 {
